/*****************************************************************************/
/*                                                                           */
/*                                Batalla Naval                              */
/*                                   servidor                                */
/*                             por Ricardo Quesada                           */
/*                                                                           */
/*****************************************************************************/
/*                                                                           */
/* Idea original por: Ricardo Quesada y Sebastian Cativa Tolosa              */
/*                                                                           */
/* Programado por:                                                           */
/*            servidor (btserver) - Ricardo Quesada                          */
/*          cliente x (xbtclient) - Ricardo Quesada                          */
/*    cliente ncurses (nbtclient) - Ricardo Quesada                          */
/*                                                                           */
/*                        Hecho en Buenos Aires, Argentina                   */
/*                                                                           */
/*****************************************************************************/
/*   Cualquier bug, mejora, etc... enviala cualquiera de estas direcciones:  */
/*                           rquesada@dc.uba.ar                              */
/*                        rquesada@era.balug.org.ar                          */
/*                            4:900/309.3@fidonet                            */
/*****************************************************************************/
/*           Distribucion del programa: GNU General Public License v2        */
/*****************************************************************************/


/* INCLUDES */
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/resource.h>
#include <unistd.h>
#include <netinet/in.h>
#include <stdio.h>
#include <syslog.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "protocol.h"                  /* definicion del protocolo */

/* DEFINES */
#define SERVERSION  "0.40"
#define BNVERH 0
#define BNVERL 40
#define MAXPLAYER 5

#define TRUE   0                      /* perhaps this is the other way */
#define FALSE  1                      /* but i dont care */

#define WHITE  0
#define BLACK  1
#define GREEN  2
#define BLUE   3
#define LBLUE  4
#define YELLOW 5
#define RED    6

#define AGUA   -2
#define NOBARCO 0                      /* PARA FUTURA IMPLEMENTACION */
#define BARCO   1 
#define TOCADO  2
#define HUNDIDO 3


/* VARIABLES GLOBALES y STRUCT y TYPEDEF y ... */

int BATNAV_TCP_PORT;            /* well known port */

typedef struct tabla_typ {
   int  x,y;
   char p[10][10];
} tabla,*tabla_ptr;

struct datos {
   int nro_tot[MAXPLAYER];     /* 1 int por jugador */
   int nro_fd[MAXPLAYER];      /* 1 file descriptor por jugador */
   int hits[MAXPLAYER];        /* how many hits has a player received */
   tabla table[MAXPLAYER];
   char names[MAXPLAYER][MAXNAMELEN]; /* other players's name  new on v0.36 */
} usuario,*usuario_ptr;

fd_set test_set, ready_set;     /* here is new on v0.35 */

int temp;                       /* variable temporal */
char tempbuf[MSGMAXLEN];        /* varible general */
/*****************************************************************************
 *                                   FUNCIONES
 *****************************************************************************/

/***************************************************************************** 
 *                           funciones TCP/IP
 ***************************************************************************/
size_t bnwrite(int fde,char *buf,char tip0,char tip1,char tip2,char jugyo ) {
   int i;
   struct protocolo proto;
   
   proto.bnptip0=tip0;
   proto.bnptip1=tip1;
   proto.bnptip2=tip2;
   proto.jugador=jugyo;
   for(i=0;i<MSGMAXLEN;i++)
     proto.bnpmsg[i]=buf[i];
   strcpy(proto.bnphead,BNPHEAD);
   return( write(fde,&proto,MAXLEN) ) ;
}

/* manda el mensaje msg a todos los que cumplan nro_tot>=yes menos a no */
void broadcast(char *msg,char tip0,int yes, int no,char tip1,char tip2,char jugyo) {         /* new on v0.34 */
   char msgnew[MSGMAXLEN];
   int i;
   
   for(i=0;i<MSGMAXLEN;i++)
     msgnew[i]=msg[i];
   
   for(i=0;i<MAXPLAYER;i++) {
      if(i!=no) {
	 if(usuario.nro_tot[i]>=yes) {
	    bnwrite(usuario.nro_fd[i],msg,tip0,tip1,tip2,jugyo);
	 }
      }
   }
}

/***************************************************************************** 
 *                              CHEQUEO DE ALGORITOMO
 ****************************************************************************/
/* points to next pos.return TRUE . FALSE in case there are no more ships */
/* this rutine uses global x,y */
int nextbarco(int num_jug) {
   if(usuario.table[num_jug].x==9) {
     usuario.table[num_jug].x=0;
     usuario.table[num_jug].y++;
   }
   else usuario.table[num_jug].x++;
   if( usuario.table[num_jug].y==10 ) return FALSE;
   else return TRUE;
}    	
/* return TRUE if in (x,y) hay barco. else FALSE */
/* this rutine uses local x,y */
int haybarco( int x, int y,int num_jug) {
   if( (x < 0) || ( x > 9 ) || (y < 0) || (y >9 ) )
     return FALSE;
   if( usuario.table[num_jug].p[x][y]==1 )
     return TRUE;
   if( usuario.table[num_jug].p[x][y]==0 )
     return FALSE;
   printf("BTSERVER: BUG or someone is cheating in haybarco()\n");
   return FALSE;
}

/* return TRUE if table is OK .else return FALSE  */
int algoritmo(int num_jug)  {
   int i,xx,yy,barcos,subarco;
   int barquitos[4];
   
   for(i=0;i<4;i++) barquitos[i]=0;
   
   /* global x,y */
   usuario.table[num_jug].x=0;
   usuario.table[num_jug].y=0;
   barcos=0;
   
   for(;;) {
      if(haybarco(usuario.table[num_jug].x,usuario.table[num_jug].y,num_jug)==FALSE) {
	 if(nextbarco(num_jug)==FALSE) {
	    if( (barquitos[3]==1) && (barquitos[2]==2) && (barquitos[1]==3) && (barquitos[0]==4) ) {
	       return TRUE;         /* tabla bien puesta */
	    }
	    else {
	       return FALSE;
	    }
	 }
      }
      else { 
	 if( haybarco(usuario.table[num_jug].x,usuario.table[num_jug].y,num_jug)==TRUE )  {
	    if( (haybarco(usuario.table[num_jug].x,usuario.table[num_jug].y-1,num_jug)==FALSE) && (haybarco(usuario.table[num_jug].x-1,usuario.table[num_jug].y,num_jug)==FALSE) ) {
	       subarco=1;
	       barcos++;
	       xx=usuario.table[num_jug].x;yy=usuario.table[num_jug].y;
	       
	       for(;;) {
		  if( (haybarco(usuario.table[num_jug].x-1,usuario.table[num_jug].y+1,num_jug)==TRUE) || (haybarco(usuario.table[num_jug].x+1,usuario.table[num_jug].y+1,num_jug)==TRUE) ) {
		     return FALSE;
		  }
		  if( (haybarco(usuario.table[num_jug].x+1,usuario.table[num_jug].y,num_jug)==FALSE) && (haybarco(usuario.table[num_jug].x,usuario.table[num_jug].y+1,num_jug)==FALSE) ) {
		     usuario.table[num_jug].x=xx;
		     usuario.table[num_jug].y=yy;          /* restauro pos */
		     barquitos[subarco-1]++; /* update cantidad de tama;os de barcos */
		     break;
		  }
		  else if( haybarco(usuario.table[num_jug].x+1,usuario.table[num_jug].y,num_jug) == TRUE ) {
		     subarco++;
		     barcos++;
		     usuario.table[num_jug].x++;
		  }          
		  else if( haybarco(usuario.table[num_jug].x,usuario.table[num_jug].y+1,num_jug) == TRUE ) {
		     usuario.table[num_jug].y++;
		     barcos++;
		     subarco++;
		  }
		  if( subarco > 4) {
		     return FALSE;
		  }
	       } /* for(;;) */
	    } /* if(haybarco(x,y-1)==FALSE) */
	 } /* if(haybarco(x,y)==TRUE) */
	 if(nextbarco(num_jug)==FALSE) {
	    if( (barquitos[3]==1) && (barquitos[2]==2) && (barquitos[1]==3) && (barquitos[0]==4) ) {
	       return TRUE;         /* tabla bien puesta */
	    }
	    else {
	       return FALSE;
	    }
	 } /* if nextbarco()==FALSE) */
      } /* else */
   } /* for(;;) */
} /* void algoritomo() */
 /***************************************************************************
 *            FUNCIONES RELACIONADAS CON LAS TABLAS 
 ***************************************************************************/
/* write to  board */
int wtable( int jug,struct protocolo *proto ) {
   int i,x,y;
   x=0;y=0;
   for(i=0;i<100;i++) {
      usuario.table[jug].p[x][y]=proto->bnpmsg[i];
      x++;
      if(x>=10) {
	 x=0;
	 y++;
      }
   }
   return( algoritmo(jug));
}
/* read from board */
int rtable( char outbuf[],int jug,char sup) {
   int i,x,y;
   x=0;y=0;
   for(i=0;i<100;i++) {
      if(sup==BNSUP)
      	outbuf[i]=usuario.table[jug].p[x][y];
      else {
	 if(usuario.table[jug].p[x][y]==1)
	   outbuf[i]=0;
	 else
	   outbuf[i]=usuario.table[jug].p[x][y];
      }
      x++;
      if(x>=10) {
	 x=0;
	 y++;
      }
   }
}
/* clean board */
int ctable( int jug) {
   int i,x,y;
   x=0;y=0;
   for(i=0;i<100;i++) {
      usuario.table[jug].p[x][y]=0;
      x++;
      if(x>=10) {
	 x=0;
	 y++;
      }
   }
}
/*****************************************************************************
 *                EL BARCO ESTA HUNDIDO ? y todo tipo de funciones
 *****************************************************************************/
int quebarco( int i,int x, int y) {
   if( (x < 0) || ( x > 9 ) || (y < 0) || (y >9 ) )
     return 0;
   return( usuario.table[i].p[x][y]);
}
/* 1 funciones recursiva para chequear si esta hundido */
int goudlr( int i,int x,int y,int xx,int yy) {
   if (quebarco(i,x,y)==2)
     goudlr(i,x+xx,y+yy,xx,yy);
   else if(quebarco(i,x,y)==1)
     temp+=1;
}
/* 1 funcion recursiva para pintar al barco hundido */
int pgoudlr( int i,int x,int y,int xx,int yy) {
   if(quebarco(i,x,y)==2) {
      usuario.table[i].p[x][y]=3;              /* 3 means hundido (sunk) */
      tempbuf[0]=(char)x;
      tempbuf[1]=(char)y;
      tempbuf[2]=BLACK;                       /* hundido */
      bnwrite(usuario.nro_fd[i],tempbuf,BNHIT,0,0,i+1);
      pgoudlr(i,x+xx,y+yy,xx,yy);
   }
}

/* i assume that quebarco(i,x,y)==2 o 1 */
int eshundido(int i,int x,int y) {
   int xx,yy;                           /* direccion a buscar */
   if(quebarco(i,x,y) <= 0)             /* IDIOTA, COMO VA A SER HUNDIDO */
     return FALSE;
   if(quebarco(i,x,y)==3)               /* estaba hundido de antes idiota */
     return TRUE;
   usuario.table[i].p[x][y]=2;            /* tocado por ahora*/
   temp=0;
   xx=-1;yy=0;goudlr(i,x,y,xx,yy);
   xx=0;yy=-1;goudlr(i,x,y,xx,yy);
   xx=1;yy=0;goudlr(i,x,y,xx,yy);
   xx=0;yy=1;goudlr(i,x,y,xx,yy);

   if(temp==0) {
      xx=-1;yy=0;pgoudlr(i,x,y,xx,yy);
      xx=0;yy=-1;pgoudlr(i,x,y-1,xx,yy);
      xx=1;yy=0;pgoudlr(i,x+1,y,xx,yy);
      xx=0;yy=1;pgoudlr(i,x,y+1,xx,yy);
      usuario.hits[i]++;
      if(usuario.hits[i]==10)  {
	 bnwrite(usuario.nro_fd[i],tempbuf,BNLOS,0,0,i+1);
	 usuario.nro_tot[i]=1;
	 usuario.hits[i]=0;
	 ctable(i);
	 sprintf(tempbuf,"Player %i is dead.",i+1); /* new on v0.32 */
	 broadcast(tempbuf,BNEXT,1,i,i+1,0,0);      /* informs that i is dead */
      }
      return TRUE;
   }
   return FALSE;
}
/***************************************************************************** 
 *                           funciones CONTROL DE CHILD Y JUEGO
 ***************************************************************************/
int play_batnav( int fd ) {
   int i,j,x,y;
   char outbuf[MSGMAXLEN];
   struct protocolo proto;

   if( read(fd,&proto,MAXLEN) < 0)
     return(-1);
   j=proto.jugador-1;

   switch (proto.bnptip0) {
    case BNREA:
      x=proto.bnptip1-1;               /* which player do you want to read */
      rtable(outbuf,x,proto.bnptip2);
      bnwrite(fd,outbuf,BNREA,x,0,j);       /* i'm sending the board */
      break;
    case BNWRI:
      for(i=0;i<MAXPLAYER;i++) {
	 if( usuario.nro_tot[i]>=3)
	   break;
      }
      if(i!=MAXPLAYER)                     /* im sorry but people is playing */
      	bnwrite(fd,outbuf,BNWAI,0,0,0);    
      else {
	 if( wtable(j,&proto)==TRUE)  {
	    usuario.nro_tot[j]=2;                    /* send board ok */
	    outbuf[0]=1;
	    broadcast(outbuf,BNCON,1,j,j+1,0,0);
	 }
	 else {
	    usuario.nro_tot[j]=1;                     /* send board not ok */
	    outbuf[0]=0;
	 }
	 bnwrite(fd,outbuf,BNWRI,0,0,0);
      }
      break;
    case BNJUG:
      for(i=0;i<MAXPLAYER;i++) {
	 if(usuario.nro_tot[i]==0)
	   break;
      }
      if(i==MAXPLAYER)                          /* im sorry, server is full*/
      	bnwrite(fd,outbuf,BNALL,i+1,0,i+1);
      else  {
	 strcpy(usuario.names[i],proto.bnpmsg);    /* new on v0.36 */
	 sprintf(tempbuf,"_%i",i+1);
	 strcat(usuario.names[i],tempbuf);
	 usuario.nro_tot[i]=1;                     /* connected */
	 usuario.nro_fd[i]=fd;
	 gethostname(outbuf,MSGMAXLEN);
	 bnwrite(fd,outbuf,BNJUG,i+1,0,i+1);
      }
      break;
    case BNVER:
      outbuf[0]=BNVERH;
      outbuf[1]=BNVERL;
      bnwrite(fd,outbuf,BNVER,0,0,0);
      break;
    case BNMSG:       /* new on v0.34 */
      sprintf(outbuf,"Player %i:",j+1);  /* from who */
      strcat(outbuf,proto.bnpmsg);
      if(proto.bnptip1==0)
      	broadcast(outbuf,BNMSG,1,j,0,0,j+1);
      else {
	 if(usuario.nro_tot[proto.bnptip1-1]>=1)
	   bnwrite(usuario.nro_fd[proto.bnptip1-1],outbuf,BNMSG,0,0,j+1);
      }
      if(proto.bnptip2==1)         
      	bnwrite(fd,outbuf,BNMSG,0,0,j+1);
      break;
    case BNTST:
      bnwrite(fd,outbuf,BNTST,0,0,0);
      break;
    case BNEXT:
      if(usuario.nro_tot[j]==4) {          /* si es el turno then fuck you */
	 for(i=proto.jugador;i<MAXPLAYER;i++) { /* pasa el turno al sig */
	    if(usuario.nro_tot[i]==3)
	      break;
	 }
	 if(i==MAXPLAYER) {                            /* llego al ultimo */
	    for(i=0;i<MAXPLAYER;i++) {
	       if(usuario.nro_tot[i]==3)
	       	 break;
	    }
	 }
	 usuario.nro_tot[i]=4;                      /* siguiente turno */
	 bnwrite(usuario.nro_fd[i],outbuf,BNTRN,0,0,i+1);
	 sleep(1);                                /* just a little delay. needed */
      }
      usuario.nro_tot[j]=0;
      usuario.hits[j]=0;
      ctable(j);  /* clear board */
      usuario.nro_fd[j]=0;
      close(fd);
      FD_CLR(fd,&test_set);
      sprintf(outbuf,"Player %i quit the game.",j+1);
      broadcast(outbuf,BNEXT,3,j,j+1,0,0); /* new on v0.35 */
      break;

    case BNHIT:
      if(usuario.nro_tot[j]==4) {                           /* is it my turn */
	 x=outbuf[0]=proto.bnpmsg[0];
	 y=outbuf[1]=proto.bnpmsg[1];
	 for(i=0;i<MAXPLAYER;i++) {
	    if(usuario.nro_tot[i]>=3){                      /* started ?*/
	       if(i!=j) {                                   /* dont shoot shooter */
		  if(usuario.table[i].p[x][y]>=1) {
		     if(eshundido(i,x,y)==FALSE) {            /* */
			outbuf[2]=RED;
			usuario.table[i].p[x][y]=2;              /* tocado */
			bnwrite(usuario.nro_fd[i],outbuf,BNHIT,0,0,i+1);
		     }
		  }
		  else if(usuario.table[i].p[x][y]<=0) {
		     outbuf[2]=GREEN;
		     usuario.table[i].p[x][y]=-1;              /* agua */
		     bnwrite(usuario.nro_fd[i],outbuf,BNHIT,0,0,i+1);
		  }
	       }
	    }
	 }
	 /* apartir del siguiente jugador */
	 usuario.nro_tot[j]=3;                       /* it is no more your turn */
	 for(i=proto.jugador;i<MAXPLAYER;i++) {
	    if(usuario.nro_tot[i]==3)
	      break;
	 }
	 if(i==MAXPLAYER) {                            /* llego al ultimo */
	    for(i=0;i<MAXPLAYER;i++) {
	       if(usuario.nro_tot[i]==3)
	       	 break;
	    }
	 }
	 if( (i==j) || (i==MAXPLAYER)) {     /* si el turno es para el mismo jugador gano */
	    bnwrite(usuario.nro_fd[i],outbuf,BNWIN,0,0,i+1);
	    sprintf(outbuf,"The game is over: Player %i is the winner.",i+1);
	    usuario.nro_tot[i]=1;
	    usuario.hits[i]=0;
	    broadcast(outbuf,BNEXT,1,i,i+1,0,0);
	    ctable(i);  /* clear board */
	 }
	 else {
	    usuario.nro_tot[i]=4;                      /* siguiente turno */
	    bnwrite(usuario.nro_fd[i],outbuf,BNTRN,0,0,i+1);
	    broadcast(outbuf,BNWHO,3,i,i+1,0,0);     /* informa de quien es */
	 }
      }
      break;
    case BNSTR:
      x=0;                          /* used as a temp variable */
      for(i=0;i<MAXPLAYER;i++) {    /* added in ver 0.29 */
	 if(usuario.nro_tot[i]>=2)
	   x++;
      }
      if(x<2)  {
	 bnwrite(usuario.nro_fd[j],outbuf,BNSOL,0,0,j);
      }
      else  {
	 for(i=0;i<MAXPLAYER;i++) {                          /* new on v0.32 */
	    outbuf[i]=usuario.nro_tot[i];                   /* show status of other players */
	    strcpy(&outbuf[5+i*MAXNAMELEN],usuario.names[i]); /* new on v0.36 */
	 }
	 for(i=0;i<MAXPLAYER;i++) {
	    if(usuario.nro_tot[i]>=2) {                     /* BUG fixed in 0.29 */
	       bnwrite(usuario.nro_fd[i],outbuf,BNSTR,0,0,i+1);
	       usuario.nro_tot[i]=3;                        /* started */
	    }
	 }
	 sleep(1);                         /* can this help. new on v0.33 */
	 for(i=0;i<MAXPLAYER;i++) {
	    if(usuario.nro_tot[i]==3) {
	       bnwrite(usuario.nro_fd[i],outbuf,BNTRN,0,0,i+1);
	       usuario.nro_tot[i]=4;
	       break;                                       /* started */
	    }
	 }
      }
      break;
    default:
      break;
   }
   return 0;
}


/*****************
 * main function *
 *****************/
void main(int argc, char *argv[]) {
   int sock,fd,client_len;
   int max_fd,i;
   struct rlimit limit_info;
   struct sockaddr_in server;
   struct sockaddr client;

   gethostname(tempbuf,MSGMAXLEN);          /* new on  v0.37 */
   
  if(argc>1) BATNAV_TCP_PORT=atoi(argv[1]);
   else BATNAV_TCP_PORT=1995;

   printf("Batalla Naval server v"SERVERSION" (c) 1995,1996\n"
	  "by Ricardo Quesada. email: rquesada@dc.uba.ar\n"
          "Running... (port=%i , name=%s , max players=%i)\n",BATNAV_TCP_PORT,tempbuf,MAXPLAYER);

   /* Set default parameters */
   for(i=0;i<MAXPLAYER;i++) {
      usuario.nro_tot[i]=0;
      usuario.nro_fd[i]=0;
      ctable(i);
   }

   if(getrlimit(RLIMIT_NOFILE,&limit_info) <0)  {
      perror("btserver: getrlimit error");
      exit(1);
   }
   
   sock=socket(AF_INET,SOCK_STREAM,0);
   if(sock <0) {
      perror("btserver: socket error");
      exit(1);
   }
   server.sin_family=AF_INET;
   server.sin_addr.s_addr=htonl(INADDR_ANY);
   server.sin_port=htons(BATNAV_TCP_PORT);
   if(bind(sock,(struct sockaddr *)&server,sizeof(server))< 0) {
      perror("btserver: bind error:");
      exit(2);
   }
   listen(sock,10);
   max_fd=sock;
   FD_ZERO(&test_set);
   FD_SET(sock,&test_set);
   while(1) {
      memcpy(&ready_set,&test_set,sizeof(test_set));
      select(max_fd+1,&ready_set,NULL,NULL,NULL);
      if(FD_ISSET(sock,&ready_set)) {
	 client_len = sizeof(client);
	 fd=accept(sock,(struct sockaddr *)&client,&client_len);
	 FD_SET(fd,&test_set);
	 if(fd>max_fd) max_fd=fd;
      }
      for(fd=0;fd<=max_fd;fd++) {
	 if((fd!=sock) && FD_ISSET(fd,&ready_set)) {
	    if(play_batnav(fd)<0) {
	       for(i=0;i<MAXPLAYER;i++) {    /* before closing...*/
		  if ( (usuario.nro_fd[i]!=sock) && (fd==usuario.nro_fd[i]) ) {
		     ctable(i);              /*...clean everything about the user */
		     usuario.nro_tot[i]=0;
		     usuario.hits[i]=0;
		     break;
		  }
	       }
	       close(fd);
	       FD_CLR(fd,&test_set);
	    }
	 } /* if(fd!=sock */
      } /* for(fd=0) */
   } /* while(1) */
}
