/* LinCycles 0.8beta by Oliver Richman. Wow! */
/* (C) Copyright 1996 Five Wheel Horse Software, Ltd. */
/* Date: 1996 02 27 */

#include <stdlib.h>
#include <stdio.h>
#include <vga.h>
#include <vgagl.h>
#include <time.h>
#include "lincycles.h"

void massprint(char ch, int x) {

	while (x-->0)
		putchar(ch);

	return;
}

void setcustompalette(int mode) {
    /* colours 0-31 are a RGB mix (bits 0 and 1 red, 2 green, 3 and 4 blue) */
    /* 32-63    black to red */
    /* 64-95    black to green */
    /* 96-127   black to yellow */
    /* 128-159  black to blue */
    /* 160-191  black to magenta */
    /* 192-223  black to cyan */
    /* 224-255  black to white */
    Palette pal;
    int i;

    for (i = 0; i < 256; i++) {
        int r, g, b;
        r = g = b = 0;
        if ((i & 32) > 0) r = (i & 31) << 1;
        if ((i & 64) > 0) g = (i & 31) << 1;
        if ((i & 128) > 0) b = (i & 31) << 1;
        if (i < 32) {
            r = (i & 3) << 4;   /* 2 bits */
            g = (i & 4) << 3;   /* 1 bit */
            b = (i & 24) << 1;  /* 2 bits */
        }
        pal.color[i].red = r;
        pal.color[i].green = g;
        pal.color[i].blue = b;
    }
    gl_setpalette(&pal);
 
}

int initfont() {
	void *font;
	font = malloc(256 * 8 * 8 * BYTESPERPIXEL);
	gl_expandfont(8, 8, white(24), gl_font8x8, font);
	gl_setfont(8, 8, font);
	return(0);
}

int clearmap() {
	int x, y;
	for (x = 0; x < MAPWIDTH; x++)
		for (y = 0; y < MAPHEIGHT; y++)
			map[x][y] = 0;
	return(0);
}

int initcycles() {

	cycle[1].stat = 0;
	cycle[1].nx = +1;
	cycle[1].ny = 0;
	cycle[1].x = (MAPWIDTH/3);
	cycle[1].y = (MAPHEIGHT/2);

	cycle[2].stat = 0;
	cycle[2].nx = -1;
	cycle[2].ny = 0;
	cycle[2].x = (MAPWIDTH/3)*2;
	cycle[2].y = (MAPHEIGHT/2);

	cycle[1].colour = P1COLOUR;
	cycle[2].colour = P1COLOUR;
	cycle[1].ocolour = P1OCOLOUR;
	cycle[2].ocolour = P2OCOLOUR;

	return(0);
}

int drawcycle(int i) {
	gl_setpixel(cycle[i].x, cycle[i].y, cycle[i].colour);
	gl_setpixel(cycle[i].ox, cycle[i].oy, cycle[i].ocolour);
	return(0);
}

int lineat(int x, int y) {

	return(map[x][y]);
}

int createbackground(void) {
	gl_clearscreen(0);
	return(0);
}

int updatescore() {
	char s[41];
	int i, x, y;
	sprintf(s, "Red: %3d:%3d  Blue: %3d:%3d  Time: %5d",
		cycle[1].x,cycle[1].y,cycle[2].x,cycle[2].y,mytime);
	gl_setwritemode(WRITEMODE_OVERWRITE);
	gl_write(0, HEIGHT - 8, s);
	return(0);
}

int update_screen() {

	drawcycle(1);
	drawcycle(2);
	updatescore();

	return(0);
}

int move(void) {
	char s[256],ch;
	long delay,collide;
	int c,i;

	/* move cycles */
	cycle[1].ox = cycle[1].x;
	cycle[2].ox = cycle[2].x;
	cycle[1].oy = cycle[1].y;
	cycle[2].oy = cycle[2].y;
	cycle[1].x += cycle[1].nx;
	cycle[2].x += cycle[2].nx;
	cycle[1].y += cycle[1].ny;
	cycle[2].y += cycle[2].ny;

	cycle[1].stat = 0;
	cycle[2].stat = 0;

	/* detect collisions. */
	if(cycle[1].x<0)
		cycle[1].stat=100;
	if(cycle[1].x>=MAPWIDTH)
		cycle[1].stat=100;
	if(cycle[2].x<0)
		cycle[2].stat=100;
	if(cycle[2].x>=MAPWIDTH)
		cycle[2].stat=100;

	if(cycle[1].y<0)
		cycle[1].stat=100;
	if(cycle[1].y>=MAPHEIGHT)
		cycle[1].stat=100;
	if(cycle[2].y<0)
		cycle[2].stat=100;
	if(cycle[2].y>=MAPHEIGHT)
		cycle[2].stat=100;


	/* detect a collision with a player's trail */
	if(map[cycle[1].x][cycle[1].y]>0)
		cycle[1].stat=200;
	if(map[cycle[2].x][cycle[2].y]>0)
		cycle[2].stat=200;

	if((cycle[1].stat>0) || (cycle[2].stat>0))
	{

	gl_setwritemode(WRITEMODE_OVERWRITE);
	gl_write(0, HEIGHT - 8, s);

	collide=0;
	if (cycle[1].stat)
		collide+=1;
	if (cycle[2].stat)	
		collide+=2;

	switch(collide) {
		case 2:
			/* player one crashed */
			sprintf(s, "LinCycle.blue crashed. Player.red wins!");
			break;
		case 1:
			/* player two crashed */
			sprintf(s, "LinCycle.red crashed. Player.blue wins!");
			break;
		case 3:
			/* both players collided */
			sprintf(s, "Both LCP crashed!@#@!@! (press any key)");
		default:
			break;
	}
	gl_setwritemode(WRITEMODE_OVERWRITE);
	gl_write(0, HEIGHT - 8, s);

	fflush(stdin);
	getchar();

	initcycles();
	clearmap();

	createbackground();

	while(vga_getkey()==-1);
        }

	map[cycle[1].x][cycle[1].y]=1;
	map[cycle[2].x][cycle[2].y]=2;
	
	return(0);
}

/* ***** */

int main(int argc, char *argv[]) {
	char s[256],ch;
	int t, mode;
	long delay,delaytime;

/* The very first thing we do is seed ourself. */
	srandom(time((time_t *)NULL)+getpid());

/* Initialize the VGA screen. */
	vga_init();

	massprint('\n',20);

	printf("LinCycles v%d.%d%s by Oliver Richman.\n",VMAJOR,VMINOR,VSTRING);
	printf("(C) Copyright 1996 Five Wheel Horse Software.\n\n");
/* Shareware beg */
	printf("This program is Shareware! The trial period is 30 days.\n");
	printf("If you like LinCycles, then please register it. For only $5,\n");
	printf("we will send you the registered version of LinCycles, plus our\n");
	printf("nifty FWH software catalogue! Source code for all programs is\n");
	printf("available. (see the read.me file for more information).\n");
	printf("PLEASE REGISTER LINCYCLES! ALSO ASK ABOUT OUR REALLY COOL PDSOFTWARE CATALOG!\n\n");
	printf("     Player 1             Player 2\n\n");
	printf("        w                    i\n");
	printf("      a + d                j + l\n");
	printf("        s                    k\n\n");

	P1COLOUR=red(31);
	P2COLOUR=red(31);
	P1OCOLOUR=red(15);
	P2OCOLOUR=blue(15);

	delaytime=DELAY;

	printf("Press enter to continue. CTRL-C to exit at any time.\n");
	getchar();
	mode=G320x200x256;

/* Load and reset the cycle drivers */
	initcycles();

/* Set the desired graphic mode. */
	vga_setmode(mode);
	gl_setcontextvga(mode);
	physicalscreen = gl_allocatecontext();
	gl_getcontext(physicalscreen);

	setcustompalette(mode);
	/* initfont() here caused trouble with planar 256 colour modes. */

	initfont();

/* Clear the player field. */
	clearmap();

	createbackground();

	for (;;) {
		int i;
		update_screen();
		move();
		update_screen();

		delay=CLOCKS_PER_SEC;
		delay/=48;
		delay*=delaytime;
		delay+=(CLOCKS_PER_SEC/25)+clock();
		while(clock()<delay)
		{

		  if((ch=vga_getkey())!=-1)
			switch(ch)
			{
				case 'w':
					cycle[1].nx=0;
					cycle[1].ny=-1;
					break;
				case 'a':
					cycle[1].nx=-1;
					cycle[1].ny=0;
					break;
				case 'd':
					cycle[1].nx=+1;
					cycle[1].ny=0;
					break;
				case 's':
					cycle[1].nx=0;
					cycle[1].ny=+1;
					break;
				case 'i':
					cycle[2].nx=0;
					cycle[2].ny=-1;
					break;
				case 'j':
					cycle[2].nx=-1;
					cycle[2].ny=0;
					break;
				case 'l':
					cycle[2].nx=+1;
					cycle[2].ny=0;
					break;
				case 'k':
					cycle[2].nx=0;
					cycle[2].ny=+1;
					break;
				default:
					break;
			}
		}
		mytime++;
	}

	vga_setmode(TEXT);	
	exit(0);
}
